//
// Created by zx on 2020/1/10.
//

#include "Verify_result.h"


/*
     * 构造函数
     * parameter:硬件限制配置参数
     */
Verify_result::Verify_result(Hardware_limit::Hardware_parameter parameter)
{
    m_hardware_parameter=parameter;
}
/*
 * 检验硬件限制
 * rotate_rect:待检验的旋转矩形
 */
Error_manager Verify_result::verify(cv::RotatedRect rotate_rect,float height,bool verify_vertex)
{
    //第一步,检验边界
    float minx=m_hardware_parameter.min_x();
    float maxx=m_hardware_parameter.max_x();
    float miny=m_hardware_parameter.min_y();
    float maxy=m_hardware_parameter.max_y();
    float center_x=rotate_rect.center.x;
    float center_y=rotate_rect.center.y;
    //左超界
    if(center_x<minx)
    {
        return Error_manager(HARDWARE_LIMIT_CENTER_X_LEFT,NORMAL,"left limit");
    }
    //右超界
    if(center_x>maxx)
    {
        return Error_manager(HARDWARE_LIMIT_CENTER_X_RIGHT,NORMAL,"right limit");
    }
    //上超界
    if(center_y>maxy)
    {
        return Error_manager(HARDWARE_LIMIT_CENTER_Y_TOP,NORMAL,"top limit");
    }
    //下超界
    if(center_y>maxy)
    {
        return Error_manager(HARDWARE_LIMIT_CENTER_Y_BOTTOM,NORMAL,"bottom limit");
    }
    //第二步,检验高度
    if(height>m_hardware_parameter.height())
    {
        char description[255]={0};
        sprintf(description,"locate height is out of range(%.2f),height:%.2f",m_hardware_parameter.height(),height);
        return Error_manager(HARDWARE_LIMIT_HEIGHT_OUT_RANGE,NORMAL,description);
    }
    //第三步,检验角度
    bool tb_theta_verify=false;
    float angle_x=rotate_rect.angle;
    for(int i=0;i<m_hardware_parameter.theta_range_size();++i)
    {
        Hardware_limit::Theta_range theta_range=m_hardware_parameter.theta_range(i);
        float min_theta=theta_range.min_theta();
        float max_theta=theta_range.max_theta();
        //角度检验, 根据plc限制
        if( angle_x>=min_theta&&angle_x<=max_theta)
        {
            tb_theta_verify = true;
        }
    }
    if(tb_theta_verify==false)
    {
        char description[255]={0};
        sprintf(description,"locate theta is out of range,theta:%.2f",angle_x);
        return Error_manager(HARDWARE_LIMIT_ANGLE_OUT_RANGE,NORMAL,description);
    }
    //是否检验顶点与栏杆的碰撞
    if(verify_vertex==false)
    {
        return SUCCESS;
    }
    //第二步,检验栏杆是否碰撞
    //创建栏杆
    const int railing_count=m_hardware_parameter.railing_parameter_size();
    if(railing_count==0)
        return SUCCESS;

    for(int i=0;i<railing_count;++i)
    {
        Hardware_limit::Railing railing_parameter=m_hardware_parameter.railing_parameter(i);
        Railing* tp_railing=new Railing(railing_parameter.pa(),railing_parameter.pb(),railing_parameter.pc(),
            railing_parameter.railing_width());
        if(tp_railing!=NULL)
        {
            Error_manager code;
            code=tp_railing->verify(rotate_rect);
            delete tp_railing;
            if(code!=SUCCESS)
            {
                return code;
            }
        }
    }
    return SUCCESS;
}