//
// Created by zx on 2019/12/9.
//

#include "plc_data.h"
#include <string.h>

Plc_data *Plc_data::g_ins = 0;
std::mutex Plc_data::g_lock;

/**
 * 获取单例句柄
 * */
Plc_data *Plc_data::get_instance(std::string ip)
{
    if (g_ins == 0)
    {
        std::lock_guard<std::mutex> lock(g_lock);
        if (g_ins == 0)
        {
            if (ip != "")
            {
                g_ins = new Plc_data(ip);
                LOG(INFO) << "plc data, handle created";
            }
            else
            {
                return 0;
            }
        }
    }
    return g_ins;
}

// void Plc_data::Release()
// {
//     if (g_ins)
//     {
//         g_ins->m_cond_exit.Notify(true);
//         LOG(INFO) << "plc data, try to exit plc send thread";
//         g_lock.lock();
//         if (g_ins->p_send_thread)
//         {
//             LOG(INFO) << "plc data, try to join plc send thread";
//             // LOG(INFO) << g_ins->p_send_thread->joinable();
//             if (g_ins->p_send_thread->joinable())
//                 g_ins->p_send_thread->join();
//             LOG(INFO) << "plc data, try to delete plc send thread";
//             delete g_ins->p_send_thread;
//             g_ins->p_send_thread = 0;
//             LOG(INFO) << "plc data, delete data send thread";
//         }
//         g_lock.unlock();

//         LOG(INFO) << "plc data, start to exit plc handle";
//         // if(g_instance->p_plc){
//         //     LOG(INFO)<<"plc data, try to delete plc handle";
//         //     delete g_instance->p_plc;
//         //     g_instance->p_plc = 0;
//         // }
//         // LOG(INFO)<<"plc data, delete plc handle";
//         LOG(INFO) << "plc data, delete instance";
//         // delete g_ins;
//         // g_ins = 0;
//         LOG(INFO) << "plc data, instance deleted";
//     }
//     else
//     {
//         LOG(WARNING) << "plc data, cannot find the instance";
//     }
// }

/**
 * 更新区域状态
 * */
void Plc_data::update_data(stateCode code, int id)
{
    // LOG(INFO) << "plc data 更新数据";
    const int length = 1;
    int offset = id * length;
    short data = 0;
    switch (code)
    {
    case stateCode::eOk:
        data = (short)1;
        break;
    case stateCode::eOutOfFrontBoundary:
        data = (short)2;
        break;
    case stateCode::eOutOfBackBoundary:
        data = (short)3;
        break;
    case stateCode::eOutOfLeftBoundary:
        data = (short)4;
        break;
    case stateCode::eOutOfRightBoundary:
        data = (short)5;
        break;
    case stateCode::eVerifyFailed:
        data = (short)6;
        break;
    case stateCode::eClusterSizeError:
        data = (short)7;
        break;
    case stateCode::eNoCar:
    default:
        data = (short)0;
        break;
    }
    std::lock_guard<std::mutex> lock(g_lock);
    ////判断越界, 更新数据
    if (offset >= 0 && offset + length <= MAX_REGIONS)
        memcpy(m_data + offset, &data, length * sizeof(short));
}

/**
 * plc更新线程，将区域状态写入plc
 * */
void Plc_data::plc_update_thread(Plc_data *p)
{
    if (p == 0)
    {
        LOG(ERROR) << "plc update thread null pointer";
        return;
    }
    while (!p->m_cond_exit.WaitFor(1))
    {
        // LOG(INFO) << "------- trying to update plc info ";
        if (p == 0)
        {
            LOG(ERROR) << "plc update thread null pointer";
            return;
        }
        // std::lock_guard<std::mutex> lock(g_lock);
        g_lock.lock();
        // 显示雷达结果
        std::cout << "result: [";
        for (int i = 0; i < MAX_REGIONS; ++i)
        {
            std::cout << p->m_data[i] << " ";
        }
        std::cout << "]" << std::endl;
        g_lock.unlock();

        // 判断plc状态
        if (p->m_plc.getConnection() && p->mb_is_ready)
        {
            // 读取门状态
            g_lock.lock();
            p->mb_is_ready = p->mb_is_ready && p->m_plc.ReadShorts(CENTRAL_CONTROLLER_DB_NUM, DOOR_STATUS_OFFSET, ELE_FENCE_COUNT, p->m_door_status);
            g_lock.unlock();

            // 写入电子围栏状态
            if (p->mb_is_ready)
            {
                short temp = 0;
                for (int i = 0; i < ELE_FENCE_COUNT; ++i)
                {
                    g_lock.lock();
                    temp = *(p->m_data + i);
                    //                temp =HTON(temp);
                    //printf("%d ---->%d\n",*(p->m_data+i), temp);
                    // 门处于开启状态，判断车辆状态，聚类数量错误则不提示
                    if (p->m_door_status[i] == 1)
                    {
                        p->mb_is_ready = p->mb_is_ready && p->m_plc.WriteShorts(ELE_FENCE_DB_NUM, ELE_FENCE_BOUNDARY_START_ADDR + i * ELE_FENCE_OFFSET, 1, &temp);
                    }
                    // 门处于关闭状态，异物入侵判断模式，只要存在识别错误情况就输入1，否则0
                    else
                    {
                        // plc是否安全位，0表示不安全，1表示安全
                        if(temp != 1 && temp != 0)
                            temp = 0;
                        else
                            temp = 1;
                        p->mb_is_ready = p->mb_is_ready && p->m_plc.WriteShorts(ELE_FENCE_DB_NUM, ELE_FENCE_SAFE_START_ADDR + i * ELE_FENCE_OFFSET, 1, &temp);
                    }
                    g_lock.unlock();
                    usleep(50 * 1000);
                }
            }
        }
        else
        {
            // 重连plc
            LOG(WARNING) << "find plc connection error, trying to reconnect.";
            g_lock.lock();
            p->m_plc.disconnect();
            // LOG(WARNING) << "find plc connection error, diconnect first.";
            if (p->m_ip_str != "")
            {
                p->mb_is_ready = p->m_plc.connect(p->m_ip_str);
                // LOG(WARNING) << "find plc connection error, trying to connect";
                if (p->mb_is_ready)
                {
                    LOG(INFO) << "successfully reconnect.";
                }
                else
                {
                    LOG(WARNING) << "failed to connect plc.";
                }
            }
            g_lock.unlock();
            usleep(p->m_update_interval_milli * 5 * 1000);
        }
        usleep(p->m_update_interval_milli * 1000);
    }
}

/**
 * 有参构造
 * */
Plc_data::Plc_data(std::string ip) : mp_update_thread(0),
                                     m_ip_str(""),
                                     mb_is_ready(0)
{
    m_ip_str = ip;
    memset(m_data, 0, MAX_REGIONS * sizeof(short));
    memset(m_door_status, 0, MAX_REGIONS * sizeof(short));
    // p_plc = new S7PLC();
    if (m_ip_str != "")
    {
        if (m_plc.connect(m_ip_str))
        {
            mb_is_ready = true;
        }
        else
        {
            LOG(ERROR) << "Plc_data instance, connect failed";
        }
    }
    else
    {
        LOG(ERROR) << "Plc_data instance, empty ip string.";
    }
    m_cond_exit.Notify(false);
    mp_update_thread = new std::thread(plc_update_thread, this);
    mp_update_thread->detach();
}

/**
 * 获取plc连接状态
 * */
bool Plc_data::get_plc_status()
{
    return mb_is_ready;
}